@@ -27,7 +27,7 @@ class ListMapTerminator(Exception):
 
 class Parser(object):
 
-    version = None
+    _version = None
     _adapter = None
 
     def __init__(self):
@@ -87,10 +87,8 @@ class Parser(object):
                     key, value = self._read_keyval()
                     self._result.headers[key] = value
 
-                elif self.version == 1 and code == b'm':
-                    if not isinstance(self._result, Call):
-                        raise ParseError('Encountered illegal method name within reply')
-
+                elif isinstance(self._result, Call) and code == b'm':
+                    self.version = 1
                     if self._result.method:
                         raise ParseError('Encountered duplicate method name definition')
 
@@ -136,11 +134,19 @@ class Parser(object):
         except ListMapTerminator:
             raise ParseError("Unhandled list/map terminator code ('Z')")
 
-    def read_version(self):
-        version = unpack('<h', self._read(2))[0]
+    @property
+    def version(self):
+        return self._version
+
+    @version.setter
+    def version(self, version):
         if version not in self._version_adapters:
             raise ParseError("Encountered unrecognized call version %r" % (version,))
         self._adapter = self._version_adapters[version](base_parser=self)
+        self._version = version
+
+    def read_version(self):
+        version = unpack('<h', self._read(2))[0]
         self.version = version
 
     def _read(self, n):
@@ -287,6 +293,7 @@ class ParserV1(object):
             fixed_length = True
 
         result = []
+        ref_idx = len(self._refs)
         self._refs.append(result)
 
         while code != b'z':
@@ -294,9 +301,9 @@ class ParserV1(object):
             code = self._read(1)
 
         if fixed_length:
-            return tuple(result)
-        else:
-            return result
+            result = self._refs[ref_idx] = tuple(result)
+
+        return result
 
     def _read_map(self):
         code = self._read(1)
@@ -501,14 +508,15 @@ class ParserV2(ParserV1):
             return super(ParserV2, self)._read_object(code)
 
     def _read_list(self, typed=False, fixed_length=False, length=None):
-        if length is 0:
-            return tuple([]) if fixed_length else []
-
         if typed:
             # read and discard list type
             self._read_object()
 
+        if length == 0:
+            return tuple([]) if fixed_length else []
+
         result = []
+        ref_idx = len(self._refs)
         self._refs.append(result)
 
         if fixed_length:
@@ -526,12 +534,12 @@ class ParserV2(ParserV1):
                     result.append(obj)
 
         if fixed_length:
-            return tuple(result)
-        else:
-            return result
+            result = self._refs[ref_idx] = tuple(result)
+
+        return result
 
     def _read_v2_string(self, code, length):
-        if length is 0:
+        if length == 0:
             return u''
         chunks = []
         while True:
